from django.core.validators import MinValueValidator, MaxValueValidator, RegexValidator
from django.db import models
from datetime import date

from main_app.custom_managers import HouseManager
from main_app.model_mixins import NameMixin, WinsMixin, ModifiedAtMixin


class House(NameMixin, WinsMixin, ModifiedAtMixin):
    motto = models.TextField(blank=True,
                             null=True)
    is_ruling = models.BooleanField(default=False)
    castle = models.CharField(max_length=80,
                              blank=True,
                              null=True)

    objects = HouseManager()

    def __str__(self):
        return self.name


class Dragon(NameMixin, WinsMixin, ModifiedAtMixin):
    class BreathChoices(models.TextChoices):
        Fire = ('Fire', 'Fire')
        Ice = ('Ice', 'Ice')
        Lightning = ('Lightning', 'Lightning')
        Unknown = ('Unknown', 'Unknown')

    power = models.DecimalField(max_digits=3,
                                decimal_places=1,
                                validators=[MinValueValidator(1.0),
                                            MaxValueValidator(10.0)],
                                default=1.0)
    breath = models.CharField(max_length=9,
                              choices=BreathChoices,
                              default=BreathChoices.Unknown)
    is_healthy = models.BooleanField(default=True)
    birth_date = models.DateField(default=date.today)
    house = models.ForeignKey(House,
                              related_name='dragons',
                              on_delete=models.CASCADE)

    def __str__(self):
        return self.name


class Quest(NameMixin, ModifiedAtMixin):
    code = models.CharField(max_length=4,
                            unique=True,
                            validators=[RegexValidator(r'^[A-Za-z#]{4}$')])
    reward = models.FloatField(default=100.0)
    start_time = models.DateTimeField()
    dragons = models.ManyToManyField(Dragon,
                                     related_name='quests')
    host = models.ForeignKey(House,
                             related_name='quests',
                             on_delete=models.CASCADE)

    def __str__(self):
        return self.name

